// Declare a class and assign it to constant `Comp`
const Comp = class {
    // Define a static property `product` and initialize it as an empty array
    static product = [];
    // Define a property `limitItems` and set its initial value to 15
    limitItems = 15;

    // Define the constructor function
    constructor() {
        // Check if there is any value in local storage under the key 'compItems'
        // If there is, parse that JSON string into an object and assign it to `Comp.product`
        // If there isn't, assign an empty array to `Comp.product`
        Comp.product = localStorage.getItem('compItems') != null ? JSON.parse(localStorage.getItem('compItems')) : [];
    }

    // Define a static method `productObj` which takes two arguments: `productObj` and `isAdd`
    static productObj(productObj, isAdd) {
        // If `isAdd` is true, add `productObj` to `product` array and return the updated array
        if (isAdd) {
            this.product.push(productObj);
            return this.product;
        }
        // If `isAdd` is false, filter out `productObj` from `product` array and return the updated array
        else {
            this.product = this.product.filter(product => product.productId != productObj.productId);
            return this.product;
        }
    }
    // Define a method `compAdd` which takes an argument `productObj`
    compAdd(productObj) {
        // Check if the number of items in `product` array is less than `limitItems`
        if (this.checkLimit()) {
            // If true, add `productObj` to local storage under the key 'compItems'
            // Then call `createCard` and `compShow` methods
            localStorage.setItem('compItems', JSON.stringify(Comp.productObj(productObj, true)));
            this.createCard();
            this.compShow();
        }
        // If false, display a message box
        else {
            msgbox(`حداکثر ${this.limitItems} محصول برای مقایسه وارد کنید`, { title: 'محدودیت مقایسه' });
        }
    }

    // Define a method `compRemove` which takes an argument `productObj`
    compRemove(productObj) {
        // Remove `productObj` from local storage under the key 'compItems'
        // Then call `createCard` and `compShow` methods
        localStorage.setItem('compItems', JSON.stringify(Comp.productObj(productObj, false)));
        this.createCard();
        this.compShow();
        // If `product` array is empty after removal, call `compHide` method
        if (Comp.product.length < 1) {
            this.compHide();
        }
    }

    // Define a method `createCard` to create a product card for each product in `product` array
    createCard() {
        let lastProductIndex = 0;
        Comp.product.forEach((product, index) => {

            if (document.querySelector('.sb_compare_items_swiper .swiper-slide:nth-child(' + (index + 1) + ').sb_compare_tray_item') != null) {
                document.querySelector('.sb_compare_items_swiper .swiper-slide:nth-child(' + (index + 1) + ').sb_compare_tray_item').classList.remove('sb_compare_tray_item');
            }
            const createTemplate = `
			<div class="sb_slab_inner" data-product-id="${product.productId}">
				<div class="sb_slab_inner_body">
					<div class="sb_slab_info">
						<div class="d-flex align-items-center">
							<div class="sb_goods_img">
								<img src="${product.productImg}" alt="">
							</div>
							<div class="sb_goods_info">
							<a href="#" class="sb_goods_title sb_font_s mt-0">${product.productTitle}</a>
							<div class="sb_good_price sb_font_s flex-column">
								<div class="sb_good_price_current">
									<div class="sb_good_price_value sb_font_m">
										${product.productCurrentPrice}
									</div>
									<div class="sb_good_price_symbol">
										<img src="${product.productCurrency}" width="18px"/>        
									</div>
								</div>
								<div class="sb_good_price_was sb_font_s mt-0">
									${product.productPriceWas} 
								</div>
							</div>
						</div>
						<div class="sb_slab_actions">
							<span class="button no-border slab-del"><i class="ico ico-times" aria-label="remove"></i></span>
						</div>
				    </div>
			    </div>
		    </div>`;
            document.querySelector('.sb_compare_items_swiper .swiper-slide:not(.sb_compare_tray_item)').innerHTML = createTemplate;
            document.querySelector('.sb_compare_items_swiper .swiper-slide:not(.sb_compare_tray_item)').classList.add('sb_compare_tray_item');
            lastProductIndex = index;
            // remove items on slider button delete
            let compObj = this;
            document.querySelectorAll('.sb_slab_actions').forEach(action => {
                action.onclick = function (e) {
                    let slideItemProductId = {
                        'productId': e.currentTarget.closest('.sb_slab_inner').dataset.productId
                    };
                    compObj.compRemove(slideItemProductId);
                    makeCompOnPageWork();
                }
            })
        });

        for (let i = Comp.product.length + 1; i <= 15; i++) {
            const emptyTemplate = `
			<div class="sb_slab_inner">
				<div class="sb_compare_tip_text">${replaceAll(i)}</div>
				</div>
			</div>
			`;
            document.querySelector('.sb_compare_items_swiper .swiper-slide:nth-child(' + i + ')').innerHTML = emptyTemplate;
            document.querySelector('.sb_compare_items_swiper .swiper-slide:nth-child(' + i + ')').classList.remove('sb_compare_tray_item');
        }
    }

    compShow() {
        document.querySelector('.sb_compare_sticky_bottom_right').classList.remove('hide');
        document.querySelector('.sb_compare_item_toolbar').classList.add('show');
    }

    compHide() {
        document.querySelector('.sb_compare_sticky_bottom_right').classList.add('hide');
        document.querySelector('.sb_compare_item_toolbar').classList.remove('show');
    }

    startCompare() {
        const arr = Comp.product.map(product => {
            return product.productId;
        });
        window.open(baseURL + "compare?productsToCompare=[" + arr + "]", "_blank");
    }

    removeAll() {
        localStorage.removeItem('compItems');
        this.compHide();
        makeCompOnPageWork();
    }
}


//add class to rows with diffrents and similarities
const highlightRows = (highlightType) => {
    let tdTextArray = [];
    $('.sb_compare_table tr:not(.sb_compare_first_row):not(.sb_compare_group_title)').each(function () {
        let tr = $(this);
        tr.find('td').each(function () {
            tdTextArray.push($(this).text().trim());
        });
        let isEqual = tdTextArray.every((element, index, arr) => element === arr[0]);
        let allEmpty = tdTextArray.every((element, index, arr) => element === "-");
        if (allEmpty) {
            tr.remove();
        } else if (highlightType === 'differences') {
            if (!isEqual) {
                tr.addClass('sb_compare_highlight_diff');
            }
        } else if (highlightType === 'similarities') {
            if (isEqual) {
                tr.addClass('sb_compare_highlight_sim');
            }
        }
        tdTextArray = [];
    });
}


//add add active class when user toggle highlights 
const toggleHighlight = (toggleClass, targetClass) => {
    $(`.${toggleClass}`).click(function (e) {
        $(`.${targetClass}`).toggleClass('active');
    });

    $(`.${toggleClass}`).change(function () {
        if (this.checked) {
            $(`.${targetClass}`).addClass('active');
        } else {
            $(`.${targetClass}`).removeClass('active');
        }
    });
}
export default {
    highlightRows,
    toggleHighlight,
    Comp
};


